//
// © 2005-2008 Marko Lindqvist
//

#include "sdksui-web.h"

#include <iostream>
#include <sstream>
#include <map>

#include "cell.h"
#include "sdksinput.h"
#include "sdksoption.h"
#include "sudoku.h"

sdks_ui_web::sdks_ui_web()
{
  sdks_ui();
}

void sdks_ui_web::solve_log(cell *pcell, const char *msg)
{
  if (get_option(OPTION_EXPLAIN)) {
    std::cout << "(" << pcell->get_x() << "," << pcell->get_y() << "):";
    std::cout << msg << std::endl;
  }
}


void sdks_ui_web::start_output()
{
  std::cout << "Content-Type:text/html;charset=UTF-8\n\n";
  std::cout << "<HTML><HEAD>";
  std::cout << "<META http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n";
  std::cout << "<style type=\"text/css\">";
  std::cout << "h1 {color: blue} h3 {color: blue}";
  std::cout << "td.in {width: 20;height: 20;text-align: center}";
  std::cout << "p.illegal {color: red}";
  std::cout << "</style>";
  std::cout << "<TITLE>Sudoku results</TITLE></HEAD><BODY>";
  std::cout << "<CENTER>" << std::endl;
}

void sdks_ui_web::finish_output()
{
  std::cout << "</CENTER></BODY></HTML>" << std::endl;
}

void sdks_ui_web::print_header(std::string msg, int size)
{
  std::cout << "<H" << size << ">" << msg << "</H" << size << ">\n";
}

void sdks_ui_web::print_msg(std::string msg)
{
  std::cout << "<P>" << msg << "<BR>\n";
}

sdks_input *sdks_ui_web::get_input(int origbase)
{
  sdks_input *input;
  int i, j;
  int base;
  int dimension;
  std::map<std::string, std::string> :: const_iterator variable;
  std::string valstr;

  if (!create_var_map()) {
    return NULL;
  }

  variable = variables.find("size");
  if (variable == variables.end()) {
    base = origbase;
  } else {
    valstr = variable->second;
    base = char_to_int(valstr.c_str()[0], 3);
    if (base < 1 || base > 5) {
      base = 3;
    }
  }
  input = new sdks_input(base);
  dimension = base*base;

  for(i = 0; i < dimension; i++) {
    for (j = 0; j < dimension; j++) {
      int val;
      int vallen;
      std::stringstream varname("", std::ios_base::out);
      varname << "CELL_" << i << "_" << j;

      variable = variables.find(varname.str());

      if (variable != variables.end()) {
        valstr = variable->second;
        vallen = valstr.length();
      } else {
	vallen = 0;
      }

      if (vallen == 0 || valstr == "?") {
	val = -1;
      } else {
	if (vallen == 1) {
	  val = char_to_int(valstr.c_str()[0], base);
	} else {
	  val = -1;
	}
	if (val < 0 || val > base*base) {
	  input->set_fail();
	  val = -2;
	}
      }

      input->set_value(j, i, val);
    }
  }

  variables.clear(); // Variables not needed any more

  return input;
}

bool sdks_ui_web::create_var_map()
{
  std::string query;

  while(std::getline(std::cin, query)) {
    size_t pos, var, val, valend;

    var = 0;
    do {
      pos = query.find('=', var);
      if (pos) {
	val = pos + 1;
	pos = query.find('&', val);
	if (!pos) {
	  valend = std::string::npos;
	} else {
	  valend = pos;
	}

	variables.insert(std::pair<std::string,std::string>(query.substr(var, val - var - 1),
							    query.substr(val, valend - val)));
      }
      var = pos + 1;
    } while (pos != std::string::npos);
  }

  return true;
}

void sdks_ui_web::print_sudoku(sudoku *puzzle)
{
  int ry, rx, y, x;
  int size = puzzle->get_size();

  std::cout << "<TABLE BORDER=\"3\">\n";

  for(ry = 0; ry < size; ry++) {
    std::cout << "<TR>";
    for (rx = 0; rx < size; rx++) {
      std::cout << "<TD><TABLE BORDER=\"1\" CELLSPACING=\"0\">\n";
      for (y = 0; y < size; y++) {
	std::cout << "  <TR>";
	for (x = 0; x < size; x++) {
	  std::cout << "<TD CLASS=\"in\">";
	  puzzle->get_cell(CT_ROW, ry*size+y, rx*size+x)->
	    print("&nbsp;", "<P class=\"illegal\">X</P>", this, size);
	  std::cout << "</TD>";
	}
	std::cout << "</TR>\n";
      }
      std::cout << "</TABLE></TD>\n";
    }
    std::cout << "</TR>";
  }

  std::cout << "</TABLE>\n";
}


void sdks_ui_web::print_sudoku_possibilities(sudoku *puzzle)
{}


void sdks_ui_web::print_method_name(const char *mname)
{
  std::cout << mname << std::endl;
}
